library saifu_qr;

import 'package:card_swiper/card_swiper.dart';
import 'package:flutter/material.dart';
import 'package:qr_flutter/qr_flutter.dart';

// ignore: must_be_immutable
class SaifuQR extends StatefulWidget {
  // List data recieved, that will be split into frama data using the amount of characters to be split by.
  final List<String> data;
  final int charPerFrame;

  // QRErrorCorrectLevel.H / M / S, higher levels ensures the qr can be read more correctly and less errors likely [more data is used]
  final int errorCorrectionalLevel;
  // This is the duration/speed of the transition from one qr to another (when multiple qr are displayed)
  final double transitionDuration;

  final bool loop;
  final bool autoplay;
  final int autoplayDelay;

  final double itemWidth;
  final double itemHeight;

  const SaifuQR({
    Key? key,
    required this.data,
    this.charPerFrame = 10,
    this.errorCorrectionalLevel = QrErrorCorrectLevel.H,
    this.transitionDuration = 500,
    this.itemHeight = 300,
    this.itemWidth = 300,
    this.loop = true,
    this.autoplay = true,
    this.autoplayDelay = 0,
  }) : super(key: key);
  @override
  State<SaifuQR> createState() => _SaifuQRState();
}

class _SaifuQRState extends State<SaifuQR> {
  // Convert all data into string
  late List<String?> frameData;

  void handleQRDAta() {
    // Retrieves data and adds them togeether into a string
    String stringData = "";
    for (int i = 0; i < widget.data.length; i++) {
      stringData = stringData + widget.data[i].toString();
    }

    // Uses Regrex Expression to break the string of that previous data into equal parts by character length (charPerFrame)
    RegExp exp = RegExp(".{1," + widget.charPerFrame.toStringAsFixed(0) + "}");
    Iterable<Match> matches = exp.allMatches(stringData);

    // This is converted into a list, and set to the frameData which is used for each QR Frame
    List<String?> list = matches.map((m) => m.group(0)).toList();
    setState(() {
      frameData = list;
    });
  }

  @override
  void initState() {
    handleQRDAta();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final customLayoutOption = CustomLayoutOption(startIndex: 0, stateCount: 5);
    customLayoutOption.addScale([1, 1, 1, 1, 1], Alignment.center);
    customLayoutOption.addTranslate([
      const Offset(0.0, 0.0),
      const Offset(0.0, 0.0),
      const Offset(0.0, 0.0),
      const Offset(0.0, 0.0),
      const Offset(0.0, 0.0),
    ]);
    customLayoutOption.addOpacity([
      0.0,
      0.0,
      0.0,
      1.0,
      0.0,
    ]);

    if (widget.data.isNotEmpty) {
      return Swiper(
        itemCount: frameData.length,
        loop: widget.loop,
        autoplay: widget.autoplay,
        duration: widget.transitionDuration.toInt(),
        itemHeight: widget.itemHeight,
        itemWidth: widget.itemWidth,
        autoplayDelay: widget.autoplayDelay,
        layout: SwiperLayout.CUSTOM,
        customLayoutOption: customLayoutOption,
        itemBuilder: (BuildContext context, int index) {
          return Center(
            child: QrImage(
              errorCorrectionLevel: widget.errorCorrectionalLevel,
              data: frameData[index].toString(),
            ),
          );
        },
      );
    } else {
      return Column(
        mainAxisSize: MainAxisSize.min,
        children: const [CircularProgressIndicator(), Text('Data List provided is null or empty')],
      );
    }
  }
}
